const { joinVoiceChannel,createAudioResource, createAudioPlayer } = require('@discordjs/voice');
const ytdl = require('ytdl-core');
const ytsr = require('ytsr');
const urlRegex=new RegExp("([a-zA-Z0-9]+://)?([a-zA-Z0-9_]+:[a-zA-Z0-9_]+@)?([a-zA-Z0-9.-]+\\.[A-Za-z]{2,4})(:[0-9]+)?(/.*)?")
const numbers = ["0⃣","1⃣","2⃣","3⃣","4⃣","5⃣","6⃣","7⃣","8⃣","9⃣"]
class Player {
	static client;
	constructor(channel,textChannel,client) {
		this.client=client
		this.channel=channel;
		this.textChannel=textChannel;
		this.playing=false;
		this.connection=joinVoiceChannel({channelId: channel.id, guildId: channel.guild.id,adapterCreator: channel.guild.voiceAdapterCreator})
		this.player=createAudioPlayer()
		this.connection.subscribe(this.player)
		this.paused=false
		this.player.addListener("stateChange", (oldOne, newOne) => {
			console.log(oldOne, newOne)
			if (newOne.status == 'playing') this.playing=true
			if (newOne.status == "idle") this.stop()
		});

	}
	stop() {
		console.log("left channel");
		this.playing=false
		this.stream.destroy()
		this.player.stop()
		this.connection.disconnect()
		this.onClose()
	}
	play(url) {
		if(!url) {
			this.textChannel.send('no url found.')
			return
		}
		if(!ytdl.validateURL(url)) {
			this.textChannel.send(`url ${url} isn't a valid YT url.`)
			return
		}
		if(this.stream) this.stream.destroy()
		this.stream = ytdl(url, { filter : 'audioonly' });
		this.resource=createAudioResource(this.stream, {inlineVolume: true})
		this.dispatcher = this.player.play(this.resource);
		this.send(`playing ${url}...`)
	}
	playpause() {
		if(this.paused) this.player.unpause()
		else this.player.pause()
		this.paused=!this.paused
	}
	send(message) {
		return this.textChannel.send(message)
	}
	onMessage(parts) {
		action=parts.shift();
		if(action=="play") this.play(parts.shift())
		else if(action=="pause"||action=="resume") {
			this.playpause();
			this.send('playpause!')
		} else if(action=="volume") {
			this.resource.volume.setVolume(parts.shift());
			this.send('volume changed!')
		} else if(action=="search") {
			this.search(parts.join(' '))
			this.send('searching...')
		} else if(action=="stop") {
			this.stop()
			this.send('stopped!')
		}
	}
	search(query) {
		ytsr(query,{limit:10}).then(res=>{
			let response=''
			res.items.forEach((item,id)=>{
				response+=`\n${numbers[id]} \`${item.title}(${item.duration} ${item.author?item.author.name:'NO AUTHOR NAME FOUND!'} ${item.id})\``
			})
			let searchCancelled=false
			this.send(response).then(message=>{
				res.items.forEach((item,id)=>{
					if(!searchCancelled) message.react(numbers[id])
				})
				message.awaitReactions({filter:(reaction,user)=>user!=this.client.user, time:30*1000,maxEmojis:1,maxUsers:1}).then(reactions=>{
					searchCancelled=true
					let reaction=reactions.first()
					if(!reaction) return
					reaction=reaction.emoji.name
					const item=res.items[parseInt(reaction)]
					this.play(item.url)
					message.reactions.removeAll()
					return true;
				})
				setTimeout(()=>message.reactions.removeAll(),30*1000)
			})
		})
	}
	static help=`play <youtube url>
volume <0-1 (default: 0.1)>
search <query>
pause
resume`
	static ONMESSAGE({message,parts}) {
		const channel=message.member.voice.channel
		if(!channel) {
			message.channel.send('Not in voice channel.')
			return
		}
		if(!channel.bot) {
			channel.bot=new Player(channel,message.channel,this.client)
			channel.bot.onClose=()=>delete channel.bot;
		}
		channel.bot.onMessage(parts)
	}
}
module.exports=client=>{
	Player.client=client;
	return Player
}
